# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.pagination import AsyncPager, SyncPager
from ...core.request_options import RequestOptions
from ...types.user_metadata import UserMetadata
from ...types.video_vector import VideoVector
from .raw_client import AsyncRawVideosClient, RawVideosClient
from .types.videos_list_request_user_metadata_value import VideosListRequestUserMetadataValue
from .types.videos_retrieve_request_embedding_option_item import VideosRetrieveRequestEmbeddingOptionItem
from .types.videos_retrieve_response import VideosRetrieveResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class VideosClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawVideosClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawVideosClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawVideosClient
        """
        return self._raw_client

    def list(
        self,
        index_id: str,
        *,
        page: typing.Optional[int] = None,
        page_limit: typing.Optional[int] = None,
        sort_by: typing.Optional[str] = None,
        sort_option: typing.Optional[str] = None,
        filename: typing.Optional[str] = None,
        duration: typing.Optional[float] = None,
        fps: typing.Optional[float] = None,
        width: typing.Optional[float] = None,
        height: typing.Optional[int] = None,
        size: typing.Optional[float] = None,
        created_at: typing.Optional[str] = None,
        updated_at: typing.Optional[str] = None,
        user_metadata: typing.Optional[typing.Dict[str, typing.Optional[VideosListRequestUserMetadataValue]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[VideoVector]:
        """
        <Info>This method will be deprecated in a future version. New implementations should use the [List indexed assets](/v1.3/api-reference/index-content/list) method.</Info>

        This method returns a list of the videos in the specified index. By default, the platform returns your videos sorted by creation date, with the newest at the top of the list.

        Parameters
        ----------
        index_id : str
            The unique identifier of the index for which the platform will retrieve the videos.

        page : typing.Optional[int]
            A number that identifies the page to retrieve.

            **Default**: `1`.

        page_limit : typing.Optional[int]
            The number of items to return on each page.

            **Default**: `10`.
            **Max**: `50`.

        sort_by : typing.Optional[str]
            The field to sort on. The following options are available:
            - `updated_at`: Sorts by the time, in the RFC 3339 format ("YYYY-MM-DDTHH:mm:ssZ"), when the item was updated.
            - `created_at`: Sorts by the time, in the RFC 3339 format ("YYYY-MM-DDTHH:mm:ssZ"), when the item was created.

            **Default**: `created_at`.

        sort_option : typing.Optional[str]
            The sorting direction. The following options are available:
            - `asc`
            - `desc`

            **Default**: `desc`.

        filename : typing.Optional[str]
            Filter by filename.

        duration : typing.Optional[float]
            Filter by duration. Expressed in seconds.

        fps : typing.Optional[float]
            Filter by frames per second.

        width : typing.Optional[float]
            Filter by width.

        height : typing.Optional[int]
            Filter by height.

        size : typing.Optional[float]
            Filter by size. Expressed in bytes.

        created_at : typing.Optional[str]
            Filter videos by the creation date and time of their associated indexing tasks, in the RFC 3339 format ("YYYY-MM-DDTHH:mm:ssZ"). The platform returns the videos whose indexing tasks were created on the specified date at or after the given time.

        updated_at : typing.Optional[str]
            This filter applies only to videos updated using the [`PUT`](/v1.3/api-reference/videos/update) method of the `/indexes/{index-id}/videos/{video-id}` endpoint. It filters videos by the last update date and time, in the RFC 3339 format ("YYYY-MM-DDTHH:mm:ssZ"). The platform returns the video indexing tasks that were last updated on the specified date at or after the given time.

        user_metadata : typing.Optional[typing.Dict[str, typing.Optional[VideosListRequestUserMetadataValue]]]
            To enable filtering by custom fields, you must first add user-defined metadata to your video by calling the [`PUT`](/v1.3/api-reference/videos/update) method of the `/indexes/:index-id/videos/:video-id` endpoint.

            Examples:
            - To filter on a string: `?category=recentlyAdded`
            - To filter on an integer: `?batchNumber=5`
            - To filter on a float: `?rating=9.3`
            - To filter on a boolean: `?needsReview=true`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[VideoVector]
            The video vectors in the specified index have successfully been retrieved.

        Examples
        --------
        from twelvelabs import TwelveLabs

        client = TwelveLabs(
            api_key="YOUR_API_KEY",
        )
        response = client.indexes.videos.list(
            index_id="6298d673f1090f1100476d4c",
            page=1,
            page_limit=10,
            sort_by="created_at",
            sort_option="desc",
            filename="01.mp4",
            duration=1.1,
            fps=1.1,
            width=1.1,
            height=1,
            size=1.1,
            created_at="2024-08-16T16:53:59Z",
            updated_at="2024-08-16T16:53:59Z",
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        return self._raw_client.list(
            index_id,
            page=page,
            page_limit=page_limit,
            sort_by=sort_by,
            sort_option=sort_option,
            filename=filename,
            duration=duration,
            fps=fps,
            width=width,
            height=height,
            size=size,
            created_at=created_at,
            updated_at=updated_at,
            user_metadata=user_metadata,
            request_options=request_options,
        )

    def retrieve(
        self,
        index_id: str,
        video_id: str,
        *,
        embedding_option: typing.Optional[
            typing.Union[
                VideosRetrieveRequestEmbeddingOptionItem, typing.Sequence[VideosRetrieveRequestEmbeddingOptionItem]
            ]
        ] = None,
        transcription: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> VideosRetrieveResponse:
        """
        <Info> This method will be deprecated in a future version. New implementations should use the [Retrieve an indexed asset](/v1.3/api-reference/index-content/retrieve) method.</Info>

        This method retrieves information about the specified video.

        Parameters
        ----------
        index_id : str
            The unique identifier of the index to which the video has been uploaded.

        video_id : str
            The unique identifier of the video to retrieve.

        embedding_option : typing.Optional[typing.Union[VideosRetrieveRequestEmbeddingOptionItem, typing.Sequence[VideosRetrieveRequestEmbeddingOptionItem]]]
            Specifies which types of embeddings to retrieve. Values: `visual`, `audio`, `transcription`. For details, see the [Embedding options](/v1.3/docs/concepts/modalities#embedding-options) section.

            <Note title="Note">
            To retrieve embeddings for a video, it must be indexed using the Marengo video understanding model. For details on enabling this model for an index, see the [Create an index](/reference/create-index) page.
            </Note>

        transcription : typing.Optional[bool]
            The parameter indicates whether to retrieve a transcription of the spoken words for the indexed video.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        VideosRetrieveResponse
            The specified video information has successfully been retrieved.

        Examples
        --------
        from twelvelabs import TwelveLabs

        client = TwelveLabs(
            api_key="YOUR_API_KEY",
        )
        client.indexes.videos.retrieve(
            index_id="6298d673f1090f1100476d4c",
            video_id="6298d673f1090f1100476d4c",
            transcription=True,
        )
        """
        _response = self._raw_client.retrieve(
            index_id,
            video_id,
            embedding_option=embedding_option,
            transcription=transcription,
            request_options=request_options,
        )
        return _response.data

    def delete(self, index_id: str, video_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        <Info>This method will be deprecated in a future version. New implementations should use the [Delete an indexed asset](/v1.3/api-reference/index-content/delete) method.</Info>

        This method deletes all the information about the specified indexed video. This action cannot be undone.

        Parameters
        ----------
        index_id : str
            The unique identifier of the index to which the video has been uploaded.

        video_id : str
            The unique identifier of the video to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from twelvelabs import TwelveLabs

        client = TwelveLabs(
            api_key="YOUR_API_KEY",
        )
        client.indexes.videos.delete(
            index_id="6298d673f1090f1100476d4c",
            video_id="6298d673f1090f1100476d4c",
        )
        """
        _response = self._raw_client.delete(index_id, video_id, request_options=request_options)
        return _response.data

    def update(
        self,
        index_id: str,
        video_id: str,
        *,
        user_metadata: typing.Optional[UserMetadata] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        <Info>This method will be deprecated in a future version. New implementations should use the [Partial update indexed asset](/v1.3/api-reference/index-content/update) method.</Info>

        This method updates one or more fields of the metadata of a video. Also, can delete a field by setting it to `null`.

        Parameters
        ----------
        index_id : str
            The unique identifier of the index to which the video has been uploaded.

        video_id : str
            The unique identifier of the video to update.

        user_metadata : typing.Optional[UserMetadata]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from twelvelabs import TwelveLabs

        client = TwelveLabs(
            api_key="YOUR_API_KEY",
        )
        client.indexes.videos.update(
            index_id="6298d673f1090f1100476d4c",
            video_id="6298d673f1090f1100476d4c",
            user_metadata={
                "category": "recentlyAdded",
                "batchNumber": 5,
                "rating": 9.3,
                "needsReview": True,
            },
        )
        """
        _response = self._raw_client.update(
            index_id, video_id, user_metadata=user_metadata, request_options=request_options
        )
        return _response.data


class AsyncVideosClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawVideosClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawVideosClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawVideosClient
        """
        return self._raw_client

    async def list(
        self,
        index_id: str,
        *,
        page: typing.Optional[int] = None,
        page_limit: typing.Optional[int] = None,
        sort_by: typing.Optional[str] = None,
        sort_option: typing.Optional[str] = None,
        filename: typing.Optional[str] = None,
        duration: typing.Optional[float] = None,
        fps: typing.Optional[float] = None,
        width: typing.Optional[float] = None,
        height: typing.Optional[int] = None,
        size: typing.Optional[float] = None,
        created_at: typing.Optional[str] = None,
        updated_at: typing.Optional[str] = None,
        user_metadata: typing.Optional[typing.Dict[str, typing.Optional[VideosListRequestUserMetadataValue]]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[VideoVector]:
        """
        <Info>This method will be deprecated in a future version. New implementations should use the [List indexed assets](/v1.3/api-reference/index-content/list) method.</Info>

        This method returns a list of the videos in the specified index. By default, the platform returns your videos sorted by creation date, with the newest at the top of the list.

        Parameters
        ----------
        index_id : str
            The unique identifier of the index for which the platform will retrieve the videos.

        page : typing.Optional[int]
            A number that identifies the page to retrieve.

            **Default**: `1`.

        page_limit : typing.Optional[int]
            The number of items to return on each page.

            **Default**: `10`.
            **Max**: `50`.

        sort_by : typing.Optional[str]
            The field to sort on. The following options are available:
            - `updated_at`: Sorts by the time, in the RFC 3339 format ("YYYY-MM-DDTHH:mm:ssZ"), when the item was updated.
            - `created_at`: Sorts by the time, in the RFC 3339 format ("YYYY-MM-DDTHH:mm:ssZ"), when the item was created.

            **Default**: `created_at`.

        sort_option : typing.Optional[str]
            The sorting direction. The following options are available:
            - `asc`
            - `desc`

            **Default**: `desc`.

        filename : typing.Optional[str]
            Filter by filename.

        duration : typing.Optional[float]
            Filter by duration. Expressed in seconds.

        fps : typing.Optional[float]
            Filter by frames per second.

        width : typing.Optional[float]
            Filter by width.

        height : typing.Optional[int]
            Filter by height.

        size : typing.Optional[float]
            Filter by size. Expressed in bytes.

        created_at : typing.Optional[str]
            Filter videos by the creation date and time of their associated indexing tasks, in the RFC 3339 format ("YYYY-MM-DDTHH:mm:ssZ"). The platform returns the videos whose indexing tasks were created on the specified date at or after the given time.

        updated_at : typing.Optional[str]
            This filter applies only to videos updated using the [`PUT`](/v1.3/api-reference/videos/update) method of the `/indexes/{index-id}/videos/{video-id}` endpoint. It filters videos by the last update date and time, in the RFC 3339 format ("YYYY-MM-DDTHH:mm:ssZ"). The platform returns the video indexing tasks that were last updated on the specified date at or after the given time.

        user_metadata : typing.Optional[typing.Dict[str, typing.Optional[VideosListRequestUserMetadataValue]]]
            To enable filtering by custom fields, you must first add user-defined metadata to your video by calling the [`PUT`](/v1.3/api-reference/videos/update) method of the `/indexes/:index-id/videos/:video-id` endpoint.

            Examples:
            - To filter on a string: `?category=recentlyAdded`
            - To filter on an integer: `?batchNumber=5`
            - To filter on a float: `?rating=9.3`
            - To filter on a boolean: `?needsReview=true`

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[VideoVector]
            The video vectors in the specified index have successfully been retrieved.

        Examples
        --------
        import asyncio

        from twelvelabs import AsyncTwelveLabs

        client = AsyncTwelveLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            response = await client.indexes.videos.list(
                index_id="6298d673f1090f1100476d4c",
                page=1,
                page_limit=10,
                sort_by="created_at",
                sort_option="desc",
                filename="01.mp4",
                duration=1.1,
                fps=1.1,
                width=1.1,
                height=1,
                size=1.1,
                created_at="2024-08-16T16:53:59Z",
                updated_at="2024-08-16T16:53:59Z",
            )
            async for item in response:
                yield item

            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        return await self._raw_client.list(
            index_id,
            page=page,
            page_limit=page_limit,
            sort_by=sort_by,
            sort_option=sort_option,
            filename=filename,
            duration=duration,
            fps=fps,
            width=width,
            height=height,
            size=size,
            created_at=created_at,
            updated_at=updated_at,
            user_metadata=user_metadata,
            request_options=request_options,
        )

    async def retrieve(
        self,
        index_id: str,
        video_id: str,
        *,
        embedding_option: typing.Optional[
            typing.Union[
                VideosRetrieveRequestEmbeddingOptionItem, typing.Sequence[VideosRetrieveRequestEmbeddingOptionItem]
            ]
        ] = None,
        transcription: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> VideosRetrieveResponse:
        """
        <Info> This method will be deprecated in a future version. New implementations should use the [Retrieve an indexed asset](/v1.3/api-reference/index-content/retrieve) method.</Info>

        This method retrieves information about the specified video.

        Parameters
        ----------
        index_id : str
            The unique identifier of the index to which the video has been uploaded.

        video_id : str
            The unique identifier of the video to retrieve.

        embedding_option : typing.Optional[typing.Union[VideosRetrieveRequestEmbeddingOptionItem, typing.Sequence[VideosRetrieveRequestEmbeddingOptionItem]]]
            Specifies which types of embeddings to retrieve. Values: `visual`, `audio`, `transcription`. For details, see the [Embedding options](/v1.3/docs/concepts/modalities#embedding-options) section.

            <Note title="Note">
            To retrieve embeddings for a video, it must be indexed using the Marengo video understanding model. For details on enabling this model for an index, see the [Create an index](/reference/create-index) page.
            </Note>

        transcription : typing.Optional[bool]
            The parameter indicates whether to retrieve a transcription of the spoken words for the indexed video.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        VideosRetrieveResponse
            The specified video information has successfully been retrieved.

        Examples
        --------
        import asyncio

        from twelvelabs import AsyncTwelveLabs

        client = AsyncTwelveLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.indexes.videos.retrieve(
                index_id="6298d673f1090f1100476d4c",
                video_id="6298d673f1090f1100476d4c",
                transcription=True,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.retrieve(
            index_id,
            video_id,
            embedding_option=embedding_option,
            transcription=transcription,
            request_options=request_options,
        )
        return _response.data

    async def delete(
        self, index_id: str, video_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> None:
        """
        <Info>This method will be deprecated in a future version. New implementations should use the [Delete an indexed asset](/v1.3/api-reference/index-content/delete) method.</Info>

        This method deletes all the information about the specified indexed video. This action cannot be undone.

        Parameters
        ----------
        index_id : str
            The unique identifier of the index to which the video has been uploaded.

        video_id : str
            The unique identifier of the video to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from twelvelabs import AsyncTwelveLabs

        client = AsyncTwelveLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.indexes.videos.delete(
                index_id="6298d673f1090f1100476d4c",
                video_id="6298d673f1090f1100476d4c",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(index_id, video_id, request_options=request_options)
        return _response.data

    async def update(
        self,
        index_id: str,
        video_id: str,
        *,
        user_metadata: typing.Optional[UserMetadata] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        <Info>This method will be deprecated in a future version. New implementations should use the [Partial update indexed asset](/v1.3/api-reference/index-content/update) method.</Info>

        This method updates one or more fields of the metadata of a video. Also, can delete a field by setting it to `null`.

        Parameters
        ----------
        index_id : str
            The unique identifier of the index to which the video has been uploaded.

        video_id : str
            The unique identifier of the video to update.

        user_metadata : typing.Optional[UserMetadata]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from twelvelabs import AsyncTwelveLabs

        client = AsyncTwelveLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.indexes.videos.update(
                index_id="6298d673f1090f1100476d4c",
                video_id="6298d673f1090f1100476d4c",
                user_metadata={
                    "category": "recentlyAdded",
                    "batchNumber": 5,
                    "rating": 9.3,
                    "needsReview": True,
                },
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(
            index_id, video_id, user_metadata=user_metadata, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class Url(UniversalBaseModel):
    """
    Provide the video via a URL.
    """

    url: str = pydantic.Field()
    """
    The publicly accessible URL of the video file.
    
    Use direct links to raw media files. Video hosting platforms and cloud storage sharing links are not supported.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .entities_create_bulk_request_entities_item import EntitiesCreateBulkRequestEntitiesItem
from .entities_list_request_sort_by import EntitiesListRequestSortBy
from .entities_list_request_status import EntitiesListRequestStatus
from .entities_list_response import EntitiesListResponse

__all__ = [
    "EntitiesCreateBulkRequestEntitiesItem",
    "EntitiesListRequestSortBy",
    "EntitiesListRequestStatus",
    "EntitiesListResponse",
]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .video_segmentation_dynamic_dynamic import VideoSegmentationDynamicDynamic
from .video_segmentation_fixed_fixed import VideoSegmentationFixedFixed


class VideoSegmentation_Dynamic(UniversalBaseModel):
    """
    Specifies how the platform divides the video into segments.
    """

    strategy: typing.Literal["dynamic"] = "dynamic"
    dynamic: VideoSegmentationDynamicDynamic

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class VideoSegmentation_Fixed(UniversalBaseModel):
    """
    Specifies how the platform divides the video into segments.
    """

    strategy: typing.Literal["fixed"] = "fixed"
    fixed: VideoSegmentationFixedFixed

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


VideoSegmentation = typing.Union[VideoSegmentation_Dynamic, VideoSegmentation_Fixed]



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from .. import core
from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pagination import AsyncPager, BaseHttpResponse, SyncPager
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..errors.bad_request_error import BadRequestError
from ..types.asset import Asset
from .types.assets_create_request_method import AssetsCreateRequestMethod
from .types.assets_list_request_asset_types_item import AssetsListRequestAssetTypesItem
from .types.assets_list_response import AssetsListResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawAssetsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self,
        *,
        page: typing.Optional[int] = None,
        page_limit: typing.Optional[int] = None,
        asset_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        asset_types: typing.Optional[
            typing.Union[AssetsListRequestAssetTypesItem, typing.Sequence[AssetsListRequestAssetTypesItem]]
        ] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[Asset]:
        """
        This method returns a list of assets in your account.

        The platform returns your assets sorted by creation date, with the newest at the top of the list.

        Parameters
        ----------
        page : typing.Optional[int]
            A number that identifies the page to retrieve.

            **Default**: `1`.

        page_limit : typing.Optional[int]
            The number of items to return on each page.

            **Default**: `10`.
            **Max**: `50`.

        asset_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Filters the response to include only assets with the specified IDs. Provide one or more asset IDs. When you specify multiple IDs, the platform returns all matching assets.

        asset_types : typing.Optional[typing.Union[AssetsListRequestAssetTypesItem, typing.Sequence[AssetsListRequestAssetTypesItem]]]
            Filters the response to include only assets of the specified types. Provide one or more asset types. When you specify multiple types, the platform returns all matching assets.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[Asset]
            The assets have been successfully retrieved.
        """
        page = page if page is not None else 1

        _response = self._client_wrapper.httpx_client.request(
            "assets",
            method="GET",
            params={
                "page": page,
                "page_limit": page_limit,
                "asset_ids": asset_ids,
                "asset_types": asset_types,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    AssetsListResponse,
                    parse_obj_as(
                        type_=AssetsListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.data
                _has_next = True
                _get_next = lambda: self.list(
                    page=page + 1,
                    page_limit=page_limit,
                    asset_ids=asset_ids,
                    asset_types=asset_types,
                    request_options=request_options,
                )
                return SyncPager(
                    has_next=_has_next, items=_items, get_next=_get_next, response=BaseHttpResponse(response=_response)
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create(
        self,
        *,
        method: AssetsCreateRequestMethod,
        file: typing.Optional[core.File] = OMIT,
        url: typing.Optional[str] = OMIT,
        filename: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[Asset]:
        """
        This method creates an asset by uploading a file to the platform. Assets are media files that you can use in downstream workflows, including indexing, analyzing video content, and creating entities.

        **Supported content**: Video, audio, and images.

        **Upload methods**:
        - **Local file**: Set the `method` parameter to `direct` and use the `file` parameter to specify the file.
        - **Publicly accessible URL**: Set the `method` parameter to `url` and use the `url` parameter to specify the URL of your file.

        **File size**: 200MB maximum for local file uploads, 4GB maximum for URL uploads.

        **Additional requirements** depend on your workflow:
        - **Search**: [Marengo requirements](/v1.3/docs/concepts/models/marengo#video-file-requirements)
        - **Video analysis**: [Pegasus requirements](/v1.3/docs/concepts/models/pegasus#input-requirements)
        - **Entity search**: [Marengo image requirements](/v1.3/docs/concepts/models/marengo#image-file-requirements)
        - **Create embeddings**: [Marengo requirements](/v1.3/docs/concepts/models/marengo#input-requirements)

        <Note title="Note">
        This endpoint is rate-limited. For details, see the [Rate limits](/v1.3/docs/get-started/rate-limits) page.
        </Note>

        Parameters
        ----------
        method : AssetsCreateRequestMethod
            Specifies the upload method for the asset. Use `direct` to upload a local file or `url` for a publicly accessible URL.

        file : typing.Optional[core.File]
            See core.File for more documentation

        url : typing.Optional[str]
            Specify this parameter to upload a file from a publicly accessible URL. This parameter is required when `method` is set to `url`.

            URL uploads have a maximum limit of 4GB.

        filename : typing.Optional[str]
            The optional filename of the asset. If not provided, the platform will determine the filename from the file or URL.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Asset]
            The asset has been successfully created.
        """
        _response = self._client_wrapper.httpx_client.request(
            "assets",
            method="POST",
            data={
                "method": method,
                "url": url,
                "filename": filename,
            },
            files={
                **({"file": file} if file is not None else {}),
            },
            request_options=request_options,
            omit=OMIT,
            force_multipart=True,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    Asset,
                    parse_obj_as(
                        type_=Asset,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def retrieve(
        self, asset_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[Asset]:
        """
        This method retrieves details about the specified asset.

        Parameters
        ----------
        asset_id : str
            The unique identifier of the asset to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[Asset]
            The asset has been successfully retrieved.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"assets/{jsonable_encoder(asset_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    Asset,
                    parse_obj_as(
                        type_=Asset,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete(self, asset_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> HttpResponse[None]:
        """
        This method deletes the specified asset. This action cannot be undone.

        Parameters
        ----------
        asset_id : str
            The unique identifier of the asset to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"assets/{jsonable_encoder(asset_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawAssetsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self,
        *,
        page: typing.Optional[int] = None,
        page_limit: typing.Optional[int] = None,
        asset_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        asset_types: typing.Optional[
            typing.Union[AssetsListRequestAssetTypesItem, typing.Sequence[AssetsListRequestAssetTypesItem]]
        ] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[Asset]:
        """
        This method returns a list of assets in your account.

        The platform returns your assets sorted by creation date, with the newest at the top of the list.

        Parameters
        ----------
        page : typing.Optional[int]
            A number that identifies the page to retrieve.

            **Default**: `1`.

        page_limit : typing.Optional[int]
            The number of items to return on each page.

            **Default**: `10`.
            **Max**: `50`.

        asset_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Filters the response to include only assets with the specified IDs. Provide one or more asset IDs. When you specify multiple IDs, the platform returns all matching assets.

        asset_types : typing.Optional[typing.Union[AssetsListRequestAssetTypesItem, typing.Sequence[AssetsListRequestAssetTypesItem]]]
            Filters the response to include only assets of the specified types. Provide one or more asset types. When you specify multiple types, the platform returns all matching assets.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[Asset]
            The assets have been successfully retrieved.
        """
        page = page if page is not None else 1

        _response = await self._client_wrapper.httpx_client.request(
            "assets",
            method="GET",
            params={
                "page": page,
                "page_limit": page_limit,
                "asset_ids": asset_ids,
                "asset_types": asset_types,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    AssetsListResponse,
                    parse_obj_as(
                        type_=AssetsListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.data
                _has_next = True

                async def _get_next():
                    return await self.list(
                        page=page + 1,
                        page_limit=page_limit,
                        asset_ids=asset_ids,
                        asset_types=asset_types,
                        request_options=request_options,
                    )

                return AsyncPager(
                    has_next=_has_next, items=_items, get_next=_get_next, response=BaseHttpResponse(response=_response)
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create(
        self,
        *,
        method: AssetsCreateRequestMethod,
        file: typing.Optional[core.File] = OMIT,
        url: typing.Optional[str] = OMIT,
        filename: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[Asset]:
        """
        This method creates an asset by uploading a file to the platform. Assets are media files that you can use in downstream workflows, including indexing, analyzing video content, and creating entities.

        **Supported content**: Video, audio, and images.

        **Upload methods**:
        - **Local file**: Set the `method` parameter to `direct` and use the `file` parameter to specify the file.
        - **Publicly accessible URL**: Set the `method` parameter to `url` and use the `url` parameter to specify the URL of your file.

        **File size**: 200MB maximum for local file uploads, 4GB maximum for URL uploads.

        **Additional requirements** depend on your workflow:
        - **Search**: [Marengo requirements](/v1.3/docs/concepts/models/marengo#video-file-requirements)
        - **Video analysis**: [Pegasus requirements](/v1.3/docs/concepts/models/pegasus#input-requirements)
        - **Entity search**: [Marengo image requirements](/v1.3/docs/concepts/models/marengo#image-file-requirements)
        - **Create embeddings**: [Marengo requirements](/v1.3/docs/concepts/models/marengo#input-requirements)

        <Note title="Note">
        This endpoint is rate-limited. For details, see the [Rate limits](/v1.3/docs/get-started/rate-limits) page.
        </Note>

        Parameters
        ----------
        method : AssetsCreateRequestMethod
            Specifies the upload method for the asset. Use `direct` to upload a local file or `url` for a publicly accessible URL.

        file : typing.Optional[core.File]
            See core.File for more documentation

        url : typing.Optional[str]
            Specify this parameter to upload a file from a publicly accessible URL. This parameter is required when `method` is set to `url`.

            URL uploads have a maximum limit of 4GB.

        filename : typing.Optional[str]
            The optional filename of the asset. If not provided, the platform will determine the filename from the file or URL.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Asset]
            The asset has been successfully created.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "assets",
            method="POST",
            data={
                "method": method,
                "url": url,
                "filename": filename,
            },
            files={
                **({"file": file} if file is not None else {}),
            },
            request_options=request_options,
            omit=OMIT,
            force_multipart=True,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    Asset,
                    parse_obj_as(
                        type_=Asset,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def retrieve(
        self, asset_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[Asset]:
        """
        This method retrieves details about the specified asset.

        Parameters
        ----------
        asset_id : str
            The unique identifier of the asset to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[Asset]
            The asset has been successfully retrieved.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"assets/{jsonable_encoder(asset_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    Asset,
                    parse_obj_as(
                        type_=Asset,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete(
        self, asset_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[None]:
        """
        This method deletes the specified asset. This action cannot be undone.

        Parameters
        ----------
        asset_id : str
            The unique identifier of the asset to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"assets/{jsonable_encoder(asset_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.http_response import AsyncHttpResponse, HttpResponse
from ...core.jsonable_encoder import jsonable_encoder
from ...core.request_options import RequestOptions


class RawTransfersClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create(
        self, integration_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[None]:
        """
        Parameters
        ----------
        integration_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"tasks/transfers/import/{jsonable_encoder(integration_id)}",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_status(
        self, integration_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[None]:
        """
        Parameters
        ----------
        integration_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"tasks/transfers/import/{jsonable_encoder(integration_id)}/status",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_logs(
        self, integration_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[None]:
        """
        Parameters
        ----------
        integration_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[None]
        """
        _response = self._client_wrapper.httpx_client.request(
            f"tasks/transfers/import/{jsonable_encoder(integration_id)}/logs",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return HttpResponse(response=_response, data=None)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawTransfersClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create(
        self, integration_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[None]:
        """
        Parameters
        ----------
        integration_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"tasks/transfers/import/{jsonable_encoder(integration_id)}",
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_status(
        self, integration_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[None]:
        """
        Parameters
        ----------
        integration_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"tasks/transfers/import/{jsonable_encoder(integration_id)}/status",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_logs(
        self, integration_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[None]:
        """
        Parameters
        ----------
        integration_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[None]
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"tasks/transfers/import/{jsonable_encoder(integration_id)}/logs",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return AsyncHttpResponse(response=_response, data=None)
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .finish_reason import FinishReason
from .stream_end_response_metadata import StreamEndResponseMetadata


class StreamEndResponse(UniversalBaseModel):
    """
    Indicates the end of the stream.
    """

    finish_reason: typing.Optional[FinishReason] = None
    metadata: typing.Optional[StreamEndResponseMetadata] = pydantic.Field(default=None)
    """
    An object containing metadata about the stream.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .finish_reason import FinishReason
from .stream_end_response_metadata import StreamEndResponseMetadata
from .stream_start_response_metadata import StreamStartResponseMetadata


class StreamAnalyzeResponse_StreamStart(UniversalBaseModel):
    """
    When the value of the `stream` parameter is set to `true`, the platform provides a streaming response in the NDJSON format.

    The stream contains three types of events:
    1. Stream start
    2. Text generation
    3. Stream end

    To integrate the response into your application, follow the guidelines below:
    - Parse each line of the response as a separate JSON object.
    - Check the `event_type` field to determine how to handle the event.
    - For `text_generation` events, process the `text` field as it arrives. Depending on your application's requirements, this may involve displaying the text incrementally, storing it for later use, or performing any tasks.
    - Use the `stream_start` and `stream_end` events to manage the lifecycle of your streaming session.
    """

    event_type: typing.Literal["stream_start"] = "stream_start"
    metadata: typing.Optional[StreamStartResponseMetadata] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class StreamAnalyzeResponse_TextGeneration(UniversalBaseModel):
    """
    When the value of the `stream` parameter is set to `true`, the platform provides a streaming response in the NDJSON format.

    The stream contains three types of events:
    1. Stream start
    2. Text generation
    3. Stream end

    To integrate the response into your application, follow the guidelines below:
    - Parse each line of the response as a separate JSON object.
    - Check the `event_type` field to determine how to handle the event.
    - For `text_generation` events, process the `text` field as it arrives. Depending on your application's requirements, this may involve displaying the text incrementally, storing it for later use, or performing any tasks.
    - Use the `stream_start` and `stream_end` events to manage the lifecycle of your streaming session.
    """

    event_type: typing.Literal["text_generation"] = "text_generation"
    text: typing.Optional[str] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class StreamAnalyzeResponse_StreamEnd(UniversalBaseModel):
    """
    When the value of the `stream` parameter is set to `true`, the platform provides a streaming response in the NDJSON format.

    The stream contains three types of events:
    1. Stream start
    2. Text generation
    3. Stream end

    To integrate the response into your application, follow the guidelines below:
    - Parse each line of the response as a separate JSON object.
    - Check the `event_type` field to determine how to handle the event.
    - For `text_generation` events, process the `text` field as it arrives. Depending on your application's requirements, this may involve displaying the text incrementally, storing it for later use, or performing any tasks.
    - Use the `stream_start` and `stream_end` events to manage the lifecycle of your streaming session.
    """

    event_type: typing.Literal["stream_end"] = "stream_end"
    finish_reason: typing.Optional[FinishReason] = None
    metadata: typing.Optional[StreamEndResponseMetadata] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


StreamAnalyzeResponse = typing.Union[
    StreamAnalyzeResponse_StreamStart, StreamAnalyzeResponse_TextGeneration, StreamAnalyzeResponse_StreamEnd
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from .....core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .....core.serialization import FieldMetadata
from .....types.embedding_data import EmbeddingData
from .tasks_create_response_status import TasksCreateResponseStatus


class TasksCreateResponse(UniversalBaseModel):
    id: typing_extensions.Annotated[str, FieldMetadata(alias="_id")] = pydantic.Field()
    """
    The unique identifier of the embedding task
    """

    status: TasksCreateResponseStatus = pydantic.Field()
    """
    The initial status of the embedding task.
    """

    data: typing.Optional[typing.List[EmbeddingData]] = pydantic.Field(default=None)
    """
    An array of embedding results when `status` is `ready`, or `null` when `status` is `processing` or `failed`.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .search_create_request_group_by import SearchCreateRequestGroupBy
from .search_create_request_operator import SearchCreateRequestOperator
from .search_create_request_query_media_type import SearchCreateRequestQueryMediaType
from .search_create_request_search_options_item import SearchCreateRequestSearchOptionsItem
from .search_create_request_transcription_options_item import SearchCreateRequestTranscriptionOptionsItem
from .search_retrieve_response import SearchRetrieveResponse
from .search_retrieve_response_page_info import SearchRetrieveResponsePageInfo

__all__ = [
    "SearchCreateRequestGroupBy",
    "SearchCreateRequestOperator",
    "SearchCreateRequestQueryMediaType",
    "SearchCreateRequestSearchOptionsItem",
    "SearchCreateRequestTranscriptionOptionsItem",
    "SearchRetrieveResponse",
    "SearchRetrieveResponsePageInfo",
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .hls_object_status import HlsObjectStatus


class HlsObject(UniversalBaseModel):
    """
    The platform returns this object only for the videos that you uploaded with the `enable_video_stream` parameter set to `true`.
    """

    video_url: typing.Optional[str] = pydantic.Field(default=None)
    """
    A string representing the URL of the video. You can then use this URL to access the stream over the <a href="https://en.wikipedia.org/wiki/HTTP_Live_Streaming" target="_blank">HLS</a> protocol.
    """

    thumbnail_urls: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    An array containing the URL of the thumbnail.
    """

    status: typing.Optional[HlsObjectStatus] = pydantic.Field(default=None)
    """
    A string representing the encoding status of the video file from its original format to a streamable format.
    
    **Values**:
    - `PROCESSING`: Video is currently being encoded and is not yet ready for streaming
    - `COMPLETE`: Encoding has successfully finished and the video is ready for streaming
    - `CANCELED`: Encoding was manually canceled before completion
    - `ERROR`: An error occurred during the encoding process
    """

    updated_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    A string indicating the date and time, in the RFC 3339 format ("YYYY-MM-DDTHH:mm:ssZ"), that the encoding status was last updated.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .videos_list_request_user_metadata_value import VideosListRequestUserMetadataValue
from .videos_list_response import VideosListResponse
from .videos_retrieve_request_embedding_option_item import VideosRetrieveRequestEmbeddingOptionItem
from .videos_retrieve_response import VideosRetrieveResponse
from .videos_retrieve_response_embedding import VideosRetrieveResponseEmbedding
from .videos_retrieve_response_embedding_video_embedding import VideosRetrieveResponseEmbeddingVideoEmbedding
from .videos_retrieve_response_system_metadata import VideosRetrieveResponseSystemMetadata

__all__ = [
    "VideosListRequestUserMetadataValue",
    "VideosListResponse",
    "VideosRetrieveRequestEmbeddingOptionItem",
    "VideosRetrieveResponse",
    "VideosRetrieveResponseEmbedding",
    "VideosRetrieveResponseEmbeddingVideoEmbedding",
    "VideosRetrieveResponseSystemMetadata",
]



# This file was auto-generated by Fern from our API Definition.

import typing

from .. import core
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.pagination import AsyncPager, SyncPager
from ..core.request_options import RequestOptions
from ..types.asset import Asset
from .raw_client import AsyncRawAssetsClient, RawAssetsClient
from .types.assets_create_request_method import AssetsCreateRequestMethod
from .types.assets_list_request_asset_types_item import AssetsListRequestAssetTypesItem

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class AssetsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawAssetsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawAssetsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawAssetsClient
        """
        return self._raw_client

    def list(
        self,
        *,
        page: typing.Optional[int] = None,
        page_limit: typing.Optional[int] = None,
        asset_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        asset_types: typing.Optional[
            typing.Union[AssetsListRequestAssetTypesItem, typing.Sequence[AssetsListRequestAssetTypesItem]]
        ] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[Asset]:
        """
        This method returns a list of assets in your account.

        The platform returns your assets sorted by creation date, with the newest at the top of the list.

        Parameters
        ----------
        page : typing.Optional[int]
            A number that identifies the page to retrieve.

            **Default**: `1`.

        page_limit : typing.Optional[int]
            The number of items to return on each page.

            **Default**: `10`.
            **Max**: `50`.

        asset_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Filters the response to include only assets with the specified IDs. Provide one or more asset IDs. When you specify multiple IDs, the platform returns all matching assets.

        asset_types : typing.Optional[typing.Union[AssetsListRequestAssetTypesItem, typing.Sequence[AssetsListRequestAssetTypesItem]]]
            Filters the response to include only assets of the specified types. Provide one or more asset types. When you specify multiple types, the platform returns all matching assets.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[Asset]
            The assets have been successfully retrieved.

        Examples
        --------
        from twelvelabs import TwelveLabs

        client = TwelveLabs(
            api_key="YOUR_API_KEY",
        )
        response = client.assets.list(
            page=1,
            page_limit=10,
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        return self._raw_client.list(
            page=page,
            page_limit=page_limit,
            asset_ids=asset_ids,
            asset_types=asset_types,
            request_options=request_options,
        )

    def create(
        self,
        *,
        method: AssetsCreateRequestMethod,
        file: typing.Optional[core.File] = OMIT,
        url: typing.Optional[str] = OMIT,
        filename: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Asset:
        """
        This method creates an asset by uploading a file to the platform. Assets are media files that you can use in downstream workflows, including indexing, analyzing video content, and creating entities.

        **Supported content**: Video, audio, and images.

        **Upload methods**:
        - **Local file**: Set the `method` parameter to `direct` and use the `file` parameter to specify the file.
        - **Publicly accessible URL**: Set the `method` parameter to `url` and use the `url` parameter to specify the URL of your file.

        **File size**: 200MB maximum for local file uploads, 4GB maximum for URL uploads.

        **Additional requirements** depend on your workflow:
        - **Search**: [Marengo requirements](/v1.3/docs/concepts/models/marengo#video-file-requirements)
        - **Video analysis**: [Pegasus requirements](/v1.3/docs/concepts/models/pegasus#input-requirements)
        - **Entity search**: [Marengo image requirements](/v1.3/docs/concepts/models/marengo#image-file-requirements)
        - **Create embeddings**: [Marengo requirements](/v1.3/docs/concepts/models/marengo#input-requirements)

        <Note title="Note">
        This endpoint is rate-limited. For details, see the [Rate limits](/v1.3/docs/get-started/rate-limits) page.
        </Note>

        Parameters
        ----------
        method : AssetsCreateRequestMethod
            Specifies the upload method for the asset. Use `direct` to upload a local file or `url` for a publicly accessible URL.

        file : typing.Optional[core.File]
            See core.File for more documentation

        url : typing.Optional[str]
            Specify this parameter to upload a file from a publicly accessible URL. This parameter is required when `method` is set to `url`.

            URL uploads have a maximum limit of 4GB.

        filename : typing.Optional[str]
            The optional filename of the asset. If not provided, the platform will determine the filename from the file or URL.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Asset
            The asset has been successfully created.

        Examples
        --------
        from twelvelabs import TwelveLabs

        client = TwelveLabs(
            api_key="YOUR_API_KEY",
        )
        client.assets.create(
            method="direct",
        )
        """
        _response = self._raw_client.create(
            method=method, file=file, url=url, filename=filename, request_options=request_options
        )
        return _response.data

    def retrieve(self, asset_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> Asset:
        """
        This method retrieves details about the specified asset.

        Parameters
        ----------
        asset_id : str
            The unique identifier of the asset to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Asset
            The asset has been successfully retrieved.

        Examples
        --------
        from twelvelabs import TwelveLabs

        client = TwelveLabs(
            api_key="YOUR_API_KEY",
        )
        client.assets.retrieve(
            asset_id="6298d673f1090f1100476d4c",
        )
        """
        _response = self._raw_client.retrieve(asset_id, request_options=request_options)
        return _response.data

    def delete(self, asset_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        This method deletes the specified asset. This action cannot be undone.

        Parameters
        ----------
        asset_id : str
            The unique identifier of the asset to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from twelvelabs import TwelveLabs

        client = TwelveLabs(
            api_key="YOUR_API_KEY",
        )
        client.assets.delete(
            asset_id="6298d673f1090f1100476d4c",
        )
        """
        _response = self._raw_client.delete(asset_id, request_options=request_options)
        return _response.data


class AsyncAssetsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawAssetsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawAssetsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawAssetsClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        page: typing.Optional[int] = None,
        page_limit: typing.Optional[int] = None,
        asset_ids: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        asset_types: typing.Optional[
            typing.Union[AssetsListRequestAssetTypesItem, typing.Sequence[AssetsListRequestAssetTypesItem]]
        ] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[Asset]:
        """
        This method returns a list of assets in your account.

        The platform returns your assets sorted by creation date, with the newest at the top of the list.

        Parameters
        ----------
        page : typing.Optional[int]
            A number that identifies the page to retrieve.

            **Default**: `1`.

        page_limit : typing.Optional[int]
            The number of items to return on each page.

            **Default**: `10`.
            **Max**: `50`.

        asset_ids : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Filters the response to include only assets with the specified IDs. Provide one or more asset IDs. When you specify multiple IDs, the platform returns all matching assets.

        asset_types : typing.Optional[typing.Union[AssetsListRequestAssetTypesItem, typing.Sequence[AssetsListRequestAssetTypesItem]]]
            Filters the response to include only assets of the specified types. Provide one or more asset types. When you specify multiple types, the platform returns all matching assets.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[Asset]
            The assets have been successfully retrieved.

        Examples
        --------
        import asyncio

        from twelvelabs import AsyncTwelveLabs

        client = AsyncTwelveLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            response = await client.assets.list(
                page=1,
                page_limit=10,
            )
            async for item in response:
                yield item

            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        return await self._raw_client.list(
            page=page,
            page_limit=page_limit,
            asset_ids=asset_ids,
            asset_types=asset_types,
            request_options=request_options,
        )

    async def create(
        self,
        *,
        method: AssetsCreateRequestMethod,
        file: typing.Optional[core.File] = OMIT,
        url: typing.Optional[str] = OMIT,
        filename: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> Asset:
        """
        This method creates an asset by uploading a file to the platform. Assets are media files that you can use in downstream workflows, including indexing, analyzing video content, and creating entities.

        **Supported content**: Video, audio, and images.

        **Upload methods**:
        - **Local file**: Set the `method` parameter to `direct` and use the `file` parameter to specify the file.
        - **Publicly accessible URL**: Set the `method` parameter to `url` and use the `url` parameter to specify the URL of your file.

        **File size**: 200MB maximum for local file uploads, 4GB maximum for URL uploads.

        **Additional requirements** depend on your workflow:
        - **Search**: [Marengo requirements](/v1.3/docs/concepts/models/marengo#video-file-requirements)
        - **Video analysis**: [Pegasus requirements](/v1.3/docs/concepts/models/pegasus#input-requirements)
        - **Entity search**: [Marengo image requirements](/v1.3/docs/concepts/models/marengo#image-file-requirements)
        - **Create embeddings**: [Marengo requirements](/v1.3/docs/concepts/models/marengo#input-requirements)

        <Note title="Note">
        This endpoint is rate-limited. For details, see the [Rate limits](/v1.3/docs/get-started/rate-limits) page.
        </Note>

        Parameters
        ----------
        method : AssetsCreateRequestMethod
            Specifies the upload method for the asset. Use `direct` to upload a local file or `url` for a publicly accessible URL.

        file : typing.Optional[core.File]
            See core.File for more documentation

        url : typing.Optional[str]
            Specify this parameter to upload a file from a publicly accessible URL. This parameter is required when `method` is set to `url`.

            URL uploads have a maximum limit of 4GB.

        filename : typing.Optional[str]
            The optional filename of the asset. If not provided, the platform will determine the filename from the file or URL.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Asset
            The asset has been successfully created.

        Examples
        --------
        import asyncio

        from twelvelabs import AsyncTwelveLabs

        client = AsyncTwelveLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.assets.create(
                method="direct",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            method=method, file=file, url=url, filename=filename, request_options=request_options
        )
        return _response.data

    async def retrieve(self, asset_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> Asset:
        """
        This method retrieves details about the specified asset.

        Parameters
        ----------
        asset_id : str
            The unique identifier of the asset to retrieve.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        Asset
            The asset has been successfully retrieved.

        Examples
        --------
        import asyncio

        from twelvelabs import AsyncTwelveLabs

        client = AsyncTwelveLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.assets.retrieve(
                asset_id="6298d673f1090f1100476d4c",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.retrieve(asset_id, request_options=request_options)
        return _response.data

    async def delete(self, asset_id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        This method deletes the specified asset. This action cannot be undone.

        Parameters
        ----------
        asset_id : str
            The unique identifier of the asset to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from twelvelabs import AsyncTwelveLabs

        client = AsyncTwelveLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.assets.delete(
                asset_id="6298d673f1090f1100476d4c",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(asset_id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ....core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ....core.serialization import FieldMetadata
from ....types.hls_object import HlsObject
from ....types.transcription_data import TranscriptionData
from .videos_retrieve_response_embedding import VideosRetrieveResponseEmbedding
from .videos_retrieve_response_system_metadata import VideosRetrieveResponseSystemMetadata


class VideosRetrieveResponse(UniversalBaseModel):
    id: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="_id")] = pydantic.Field(default=None)
    """
    The unique identifier of the video.
    """

    created_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    A string indicating the date and time, in the RFC 3339 format ("YYYY-MM-DDTHH:mm:ssZ"), that the video indexing task was created.
    """

    updated_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    A string indicating the date and time, in the RFC 3339 format ("YYYY-MM-DDTHH:mm:ssZ"), that the corresponding video indexing task was last updated. The platform updates this field every time the corresponding video indexing task transitions to a different state.
    """

    indexed_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    A string indicating the date and time, in the RFC 3339 format ("YYYY-MM-DDTHH:mm:ssZ"), that the video indexing task has been completed.
    """

    system_metadata: typing.Optional[VideosRetrieveResponseSystemMetadata] = pydantic.Field(default=None)
    """
    System-generated metadata about the video.
    """

    user_metadata: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = pydantic.Field(default=None)
    """
    User-generated metadata about the video.
    """

    hls: typing.Optional[HlsObject] = None
    embedding: typing.Optional[VideosRetrieveResponseEmbedding] = pydantic.Field(default=None)
    """
    Contains the embedding and the associated information. The platform returns this field when the `embedding_option` parameter is specified in the request.
    """

    transcription: typing.Optional[TranscriptionData] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pagination import AsyncPager, BaseHttpResponse, SyncPager
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from ..errors.bad_request_error import BadRequestError
from ..errors.forbidden_error import ForbiddenError
from ..errors.internal_server_error import InternalServerError
from ..errors.not_found_error import NotFoundError
from ..types.chunk_info import ChunkInfo
from ..types.completed_chunk import CompletedChunk
from ..types.create_asset_upload_response import CreateAssetUploadResponse
from ..types.get_upload_status_response import GetUploadStatusResponse
from ..types.incomplete_upload_summary import IncompleteUploadSummary
from ..types.list_incomplete_uploads_response import ListIncompleteUploadsResponse
from ..types.report_chunk_batch_response import ReportChunkBatchResponse
from ..types.request_additional_presigned_ur_ls_response import RequestAdditionalPresignedUrLsResponse
from .types.create_asset_upload_request_type import CreateAssetUploadRequestType

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawMultipartUploadClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list_incomplete_uploads(
        self,
        *,
        page: typing.Optional[int] = None,
        page_limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[IncompleteUploadSummary]:
        """
        This method returns a list of all incomplete multipart upload sessions in your account.

        Parameters
        ----------
        page : typing.Optional[int]
            A number that identifies the page to retrieve.

            **Default**: `1`.

        page_limit : typing.Optional[int]
            The number of items to return on each page.

            **Default**: `10`.
            **Max**: `50`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[IncompleteUploadSummary]
            The list of incomplete upload sessions has been successfully retrieved.
        """
        page = page if page is not None else 1

        _response = self._client_wrapper.httpx_client.request(
            "assets/multipart-uploads",
            method="GET",
            params={
                "page": page,
                "page_limit": page_limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListIncompleteUploadsResponse,
                    parse_obj_as(
                        type_=ListIncompleteUploadsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.data
                _has_next = True
                _get_next = lambda: self.list_incomplete_uploads(
                    page=page + 1,
                    page_limit=page_limit,
                    request_options=request_options,
                )
                return SyncPager(
                    has_next=_has_next, items=_items, get_next=_get_next, response=BaseHttpResponse(response=_response)
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def create(
        self,
        *,
        filename: str,
        type: CreateAssetUploadRequestType,
        total_size: int,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[CreateAssetUploadResponse]:
        """
        This method creates a multipart upload session.

        **Supported content**: Video and audio

        **File size**: 4GB maximum.

        **Additional requirements** depend on your workflow:
        - **Search**: [Marengo requirements](/v1.3/docs/concepts/models/marengo#video-file-requirements)
        - **Video analysis**: [Pegasus requirements](/v1.3/docs/concepts/models/pegasus#input-requirements)
        - **Create embeddings**: [Marengo requirements](/v1.3/docs/concepts/models/marengo#input-requirements)

        Parameters
        ----------
        filename : str
            The original file name of the asset.

        type : CreateAssetUploadRequestType
            The type of asset you want to upload.

        total_size : int
            The total size of the file in bytes. The platform uses this value to:
            - Calculate the optimal chunk size.
            - Determine the total number of chunks required
            - Generate the initial set of presigned URLs

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[CreateAssetUploadResponse]
            The multipart upload session has been successfully created.
        """
        _response = self._client_wrapper.httpx_client.request(
            "assets/multipart-uploads",
            method="POST",
            json={
                "filename": filename,
                "type": type,
                "total_size": total_size,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateAssetUploadResponse,
                    parse_obj_as(
                        type_=CreateAssetUploadResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_status(
        self,
        upload_id: str,
        *,
        page: typing.Optional[int] = None,
        page_limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[ChunkInfo]:
        """
        This method provides information about an upload session, including its current status, chunk-level progress, and completion state.

        Use this method to:
        - Verify upload completion (`status` = `completed`)
        - Identify any failed chunks that require a retry
        - Monitor the upload progress by comparing `uploaded_size` with `total_size`
        - Determine if the session has expired
        - Retrieve the status information for each chunk

        You must call this method after reporting chunk completion to confirm the upload has transitioned to the `completed` status before using the asset.

        Parameters
        ----------
        upload_id : str
            The unique identifier of the upload session.

        page : typing.Optional[int]
            A number that identifies the page to retrieve.

            **Default**: `1`.

        page_limit : typing.Optional[int]
            The number of items to return on each page.

            **Default**: `10`.
            **Max**: `50`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[ChunkInfo]
            The status of your upload session has been successfully retrieved.
        """
        page = page if page is not None else 1

        _response = self._client_wrapper.httpx_client.request(
            f"assets/multipart-uploads/{jsonable_encoder(upload_id)}",
            method="GET",
            params={
                "page": page,
                "page_limit": page_limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    GetUploadStatusResponse,
                    parse_obj_as(
                        type_=GetUploadStatusResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.uploaded_chunks
                _has_next = True
                _get_next = lambda: self.get_status(
                    upload_id,
                    page=page + 1,
                    page_limit=page_limit,
                    request_options=request_options,
                )
                return SyncPager(
                    has_next=_has_next, items=_items, get_next=_get_next, response=BaseHttpResponse(response=_response)
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def report_chunk_batch(
        self,
        upload_id: str,
        *,
        completed_chunks: typing.Sequence[CompletedChunk],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ReportChunkBatchResponse]:
        """
        This method reports successfully uploaded chunks to the platform. The platform finalizes the upload after you report all chunks.


        For optimal performance, report chunks in batches and in any order.

        Parameters
        ----------
        upload_id : str
            The unique identifier of the upload session.

        completed_chunks : typing.Sequence[CompletedChunk]
            The list of chunks successfully uploaded that you're reporting to the platform. Report only after receiving an ETag.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ReportChunkBatchResponse]
            The completion of this batch has been successfully reported.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"assets/multipart-uploads/{jsonable_encoder(upload_id)}",
            method="POST",
            json={
                "completed_chunks": convert_and_respect_annotation_metadata(
                    object_=completed_chunks, annotation=typing.Sequence[CompletedChunk], direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ReportChunkBatchResponse,
                    parse_obj_as(
                        type_=ReportChunkBatchResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_additional_presigned_urls(
        self, upload_id: str, *, start: int, count: int, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[RequestAdditionalPresignedUrLsResponse]:
        """
        This method generates new presigned URLs for specific chunks that require uploading. Use this endpoint in the following situations:
        - Your initial URLs have expired (URLs expire after one hour).
        - The initial set of presigned URLs does not include URLs for all chunks.
        - You need to retry failed chunk uploads with new URLs.
        To specify which chunks need URLs, use the `start` and `count` parameters. For example, to generate URLs for chunks 21 to 30, use `start=21` and `count=10`.
        The response will provide new URLs, each with a fresh expiration time of one hour.

        Parameters
        ----------
        upload_id : str
            The unique identifier of the upload session.

        start : int
            The index of the first chunk number to generate URLs for. Chunks are numbered from 1.

        count : int
            The number of presigned URLs to generate starting from the index. You can request a maximum of 50 URLs in a single API call.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[RequestAdditionalPresignedUrLsResponse]
            Additional presigned URLs have been successfully generated.
        """
        _response = self._client_wrapper.httpx_client.request(
            f"assets/multipart-uploads/{jsonable_encoder(upload_id)}/presigned-urls",
            method="POST",
            json={
                "start": start,
                "count": count,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    RequestAdditionalPresignedUrLsResponse,
                    parse_obj_as(
                        type_=RequestAdditionalPresignedUrLsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawMultipartUploadClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list_incomplete_uploads(
        self,
        *,
        page: typing.Optional[int] = None,
        page_limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[IncompleteUploadSummary]:
        """
        This method returns a list of all incomplete multipart upload sessions in your account.

        Parameters
        ----------
        page : typing.Optional[int]
            A number that identifies the page to retrieve.

            **Default**: `1`.

        page_limit : typing.Optional[int]
            The number of items to return on each page.

            **Default**: `10`.
            **Max**: `50`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[IncompleteUploadSummary]
            The list of incomplete upload sessions has been successfully retrieved.
        """
        page = page if page is not None else 1

        _response = await self._client_wrapper.httpx_client.request(
            "assets/multipart-uploads",
            method="GET",
            params={
                "page": page,
                "page_limit": page_limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    ListIncompleteUploadsResponse,
                    parse_obj_as(
                        type_=ListIncompleteUploadsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.data
                _has_next = True

                async def _get_next():
                    return await self.list_incomplete_uploads(
                        page=page + 1,
                        page_limit=page_limit,
                        request_options=request_options,
                    )

                return AsyncPager(
                    has_next=_has_next, items=_items, get_next=_get_next, response=BaseHttpResponse(response=_response)
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def create(
        self,
        *,
        filename: str,
        type: CreateAssetUploadRequestType,
        total_size: int,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[CreateAssetUploadResponse]:
        """
        This method creates a multipart upload session.

        **Supported content**: Video and audio

        **File size**: 4GB maximum.

        **Additional requirements** depend on your workflow:
        - **Search**: [Marengo requirements](/v1.3/docs/concepts/models/marengo#video-file-requirements)
        - **Video analysis**: [Pegasus requirements](/v1.3/docs/concepts/models/pegasus#input-requirements)
        - **Create embeddings**: [Marengo requirements](/v1.3/docs/concepts/models/marengo#input-requirements)

        Parameters
        ----------
        filename : str
            The original file name of the asset.

        type : CreateAssetUploadRequestType
            The type of asset you want to upload.

        total_size : int
            The total size of the file in bytes. The platform uses this value to:
            - Calculate the optimal chunk size.
            - Determine the total number of chunks required
            - Generate the initial set of presigned URLs

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[CreateAssetUploadResponse]
            The multipart upload session has been successfully created.
        """
        _response = await self._client_wrapper.httpx_client.request(
            "assets/multipart-uploads",
            method="POST",
            json={
                "filename": filename,
                "type": type,
                "total_size": total_size,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    CreateAssetUploadResponse,
                    parse_obj_as(
                        type_=CreateAssetUploadResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_status(
        self,
        upload_id: str,
        *,
        page: typing.Optional[int] = None,
        page_limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[ChunkInfo]:
        """
        This method provides information about an upload session, including its current status, chunk-level progress, and completion state.

        Use this method to:
        - Verify upload completion (`status` = `completed`)
        - Identify any failed chunks that require a retry
        - Monitor the upload progress by comparing `uploaded_size` with `total_size`
        - Determine if the session has expired
        - Retrieve the status information for each chunk

        You must call this method after reporting chunk completion to confirm the upload has transitioned to the `completed` status before using the asset.

        Parameters
        ----------
        upload_id : str
            The unique identifier of the upload session.

        page : typing.Optional[int]
            A number that identifies the page to retrieve.

            **Default**: `1`.

        page_limit : typing.Optional[int]
            The number of items to return on each page.

            **Default**: `10`.
            **Max**: `50`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[ChunkInfo]
            The status of your upload session has been successfully retrieved.
        """
        page = page if page is not None else 1

        _response = await self._client_wrapper.httpx_client.request(
            f"assets/multipart-uploads/{jsonable_encoder(upload_id)}",
            method="GET",
            params={
                "page": page,
                "page_limit": page_limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    GetUploadStatusResponse,
                    parse_obj_as(
                        type_=GetUploadStatusResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.uploaded_chunks
                _has_next = True

                async def _get_next():
                    return await self.get_status(
                        upload_id,
                        page=page + 1,
                        page_limit=page_limit,
                        request_options=request_options,
                    )

                return AsyncPager(
                    has_next=_has_next, items=_items, get_next=_get_next, response=BaseHttpResponse(response=_response)
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def report_chunk_batch(
        self,
        upload_id: str,
        *,
        completed_chunks: typing.Sequence[CompletedChunk],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ReportChunkBatchResponse]:
        """
        This method reports successfully uploaded chunks to the platform. The platform finalizes the upload after you report all chunks.


        For optimal performance, report chunks in batches and in any order.

        Parameters
        ----------
        upload_id : str
            The unique identifier of the upload session.

        completed_chunks : typing.Sequence[CompletedChunk]
            The list of chunks successfully uploaded that you're reporting to the platform. Report only after receiving an ETag.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ReportChunkBatchResponse]
            The completion of this batch has been successfully reported.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"assets/multipart-uploads/{jsonable_encoder(upload_id)}",
            method="POST",
            json={
                "completed_chunks": convert_and_respect_annotation_metadata(
                    object_=completed_chunks, annotation=typing.Sequence[CompletedChunk], direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ReportChunkBatchResponse,
                    parse_obj_as(
                        type_=ReportChunkBatchResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_additional_presigned_urls(
        self, upload_id: str, *, start: int, count: int, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[RequestAdditionalPresignedUrLsResponse]:
        """
        This method generates new presigned URLs for specific chunks that require uploading. Use this endpoint in the following situations:
        - Your initial URLs have expired (URLs expire after one hour).
        - The initial set of presigned URLs does not include URLs for all chunks.
        - You need to retry failed chunk uploads with new URLs.
        To specify which chunks need URLs, use the `start` and `count` parameters. For example, to generate URLs for chunks 21 to 30, use `start=21` and `count=10`.
        The response will provide new URLs, each with a fresh expiration time of one hour.

        Parameters
        ----------
        upload_id : str
            The unique identifier of the upload session.

        start : int
            The index of the first chunk number to generate URLs for. Chunks are numbered from 1.

        count : int
            The number of presigned URLs to generate starting from the index. You can request a maximum of 50 URLs in a single API call.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[RequestAdditionalPresignedUrLsResponse]
            Additional presigned URLs have been successfully generated.
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"assets/multipart-uploads/{jsonable_encoder(upload_id)}/presigned-urls",
            method="POST",
            json={
                "start": start,
                "count": count,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    RequestAdditionalPresignedUrLsResponse,
                    parse_obj_as(
                        type_=RequestAdditionalPresignedUrLsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

from .. import core
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.search_results import SearchResults
from .raw_client import AsyncRawSearchClient, RawSearchClient
from .types.search_create_request_group_by import SearchCreateRequestGroupBy
from .types.search_create_request_operator import SearchCreateRequestOperator
from .types.search_create_request_query_media_type import SearchCreateRequestQueryMediaType
from .types.search_create_request_search_options_item import SearchCreateRequestSearchOptionsItem
from .types.search_create_request_transcription_options_item import SearchCreateRequestTranscriptionOptionsItem
from .types.search_retrieve_response import SearchRetrieveResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class SearchClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawSearchClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawSearchClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawSearchClient
        """
        return self._raw_client

    def create(
        self,
        *,
        index_id: str,
        search_options: typing.List[SearchCreateRequestSearchOptionsItem],
        query_media_type: typing.Optional[SearchCreateRequestQueryMediaType] = OMIT,
        query_media_url: typing.Optional[str] = OMIT,
        query_media_file: typing.Optional[core.File] = OMIT,
        query_text: typing.Optional[str] = OMIT,
        transcription_options: typing.Optional[typing.List[SearchCreateRequestTranscriptionOptionsItem]] = OMIT,
        group_by: typing.Optional[SearchCreateRequestGroupBy] = OMIT,
        operator: typing.Optional[SearchCreateRequestOperator] = OMIT,
        page_limit: typing.Optional[int] = OMIT,
        filter: typing.Optional[str] = OMIT,
        include_user_metadata: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SearchResults:
        """
        Use this endpoint to search for relevant matches in an index using text, media, or a combination of both as your query.
        
        **Text queries**:
        - Use the `query_text` parameter to specify your query.
        
        **Media queries**:
        - Set the `query_media_type` parameter to the corresponding media type (example: `image`).
        - Provide up to 10 images by specifying the following parameters multiple times:
          - `query_media_url`: Publicly accessible URL of your media file.
          - `query_media_file`: Local media file.
        **Composed text and media queries**:
        - Use the `query_text` parameter for your text query.
        - Set `query_media_type` to `image`.
        - Provide up to 10 images by specifying the `query_media_url` and `query_media_file` parameters multiple times.
        
        **Entity search** (beta):
        - To find a specific person in your videos, enclose the unique identifier of the entity you want to find in the `query_text` parameter.
        
        <Note title="Notes">
        - When using images in your search queries (either as media queries or in composed searches), ensure your image files meet the [requirements](/v1.3/docs/concepts/models/marengo#image-file-requirements).
        - This endpoint is rate-limited. For details, see the [Rate limits](/v1.3/docs/get-started/rate-limits) page.
        </Note>
        
        Parameters
        ----------
        index_id : str
            The unique identifier of the index to search.
        
        search_options : typing.List[SearchCreateRequestSearchOptionsItem]
            Specifies the modalities the video understanding model uses to find relevant information.
            
            Available options:
            - `visual`: Searches visual content.
            - `audio`: Searches non-speech audio.
            - `transcription`: Spoken words
            
            <Note title="Note">
            - You can specify multiple search options in conjunction with the [`operator`](/v1.3/api-reference/any-to-video-search/make-search-request#request.body.operator.operator) parameter described below to broaden or narrow your search. For example, to search using both visual and non-speech audio content, include this parameter two times in the request as shown below:
              ```JSON
              --form search_options=visual \
              --form search_options=audio \
              --form search_options=transcription \
              ```
            </Note>
            
            For guidance, see the [Search options](/v1.3/docs/concepts/modalities#search-options) section.
        
        query_media_type : typing.Optional[SearchCreateRequestQueryMediaType]
            The type of media you wish to use. This parameter is required for media queries. For example, to perform an image-based search, set this parameter to `image`. Use `query_text` together with this parameter when you want to perform a composed image+text search.
        
        query_media_url : typing.Optional[str]
            The publicly accessible URL of a media file to use as a query. This parameter is required for media queries if `query_media_file` is not provided.
            
            You can provide up to 10 images by specifying this parameter multiple times (Marengo 3.0 only):
            ```
            --form query_media_url=https://example.com/image1.jpg \
            --form query_media_url=https://example.com/image2.jpg
            ```
        
        query_media_file : typing.Optional[core.File]
            See core.File for more documentation
        
        query_text : typing.Optional[str]
            The text query to search for. This parameter is required for text queries. Note that the platform supports full natural language-based search. You can use this parameter together with `query_media_type` and `query_media_url` or `query_media_file` to perform a composed image+text search.
            
            If you're using the Entity Search feature to search for specific persons in your video content, you must enclose the unique identifier of your entity between the `<@` and `>` markers. For example, to search for an entity with the ID `entity123`, use `<@entity123> is walking` as your query.
            
            
            Marengo supports up to 500 tokens per query.
        
        transcription_options : typing.Optional[typing.List[SearchCreateRequestTranscriptionOptionsItem]]
            Specifies how the platform matches your text query with the words spoken in the video. This parameter applies only when the `search_options` parameter contains the `transcription` value.
            
            Available options:
            - `lexical`: Exact word matching
            - `semantic`: Meaning-based matching
            
            For details on when to use each option, see the [Transcription options](/v1.3/docs/concepts/modalities#transcription-options) section.
            
            **Default**: `["lexical", "semantic"]`.
        
        group_by : typing.Optional[SearchCreateRequestGroupBy]
            Use this parameter to group or ungroup items in a response. It can take one of the following values:
            - `video`:  The platform will group the matching video clips in the response by video.
            - `clip`: The matching video clips in the response will not be grouped.
            
            **Default:** `clip`
        
        operator : typing.Optional[SearchCreateRequestOperator]
            Combines multiple search options using `or` or `and`. Use `and` to find segments matching all search options. Use `or` to find segments matching any search option. For detailed guidance on using this parameter, see the [Combine multiple modalities](/v1.3/docs/concepts/modalities#combine-multiple-modalities) section.
            
            **Default**: `or`.
        
        page_limit : typing.Optional[int]
            The number of items to return on each page. When grouping by video, this parameter represents the number of videos per page. Otherwise, it represents the maximum number of video clips per page.
            
            **Max**: `50`.
        
        filter : typing.Optional[str]
            Specifies a stringified JSON object to filter your search results. Supports both system-generated metadata (example: video ID, duration) and user-defined metadata.
            
            **Syntax for filtering**
            
            The following table describes the supported data types, operators, and filter syntax:
            
            | Data type | Operator | Description | Syntax |
            |:----------|:---------|:------------|:-------|
            | String | `=` | Matches results equal to the specified value. | `{"field": "value"}`
            | Array of strings | `=` | Matches results with any value in the specified array. Supported only for `id`. | `{"id": ["value1", "value2"]}` |
            | Numeric (integer, float) | `=`, `lte`, `gte` | Matches results equal to or within a range of the specified value. | `{"field": number}` or `{"field": { "gte": number, "lte": number }}` |
            | Boolean | `=` | Matches results equal to the specified boolean value. | `{"field": true}` or `{"field": false}`. |
            
            <br/>
            **System-generated metadata**
            
            The table below describes the system-generated metadata available for filtering your  search results:
            
            | Field name | Description | Type | Example |
            |:-----------|:------------|:-----|:--------|
            | `id` | Filters by specific video IDs. | Array of strings | `{"id": ["67cec9caf45d9b64a58340fc", "67cec9baf45d9b64a58340fa"]}`. |
            | `duration` | Filters based on the duration of the video containing the segment that matches your query. | Number or object with `gte` and `lte` | `{"duration": 600}` or `{"duration": { "gte": 600, "lte": 800 }}` |
            | `width` | Filters by video width (in pixels). | Number or object with `gte` and `lte` | `{"width": 1920}` or `{"width": { "gte": 1280, "lte": 1920}}` |
            | `height` | Filters by video height (in pixels). | Number or object with `gte` and `lte`. | `{"height": 1080}` or `{"height": { "gte": 720, "lte": 1080 }}`. |
            | `size` | Filters by video size (in bytes) | Number or object with `gte` and `lte`. | `{"size": 1048576}` or `{"size": { "gte": 1048576, "lte": 5242880}}` |
            | `filename` | Filters by the exact file name. | String | `{"filename": "Animal Encounters part 1"}` |
            
            <br/>
            **User-defined metadata**
            
            To filter by user-defined metadata:
            1. Add metadata to your video by calling the [`PUT`](/v1.3/api-reference/videos/update) method of the `/indexes/:index-id/videos/:video-id` endpoint
            2. Reference the custom field in your filter object. For example, to filter videos where a custom field named `needsReview` of type boolean is `true`, use `{"needs_review": true}`.
            
            For more details and examples, see the [Filter search results](/v1.3/docs/guides/search/filtering) page.
        
        include_user_metadata : typing.Optional[bool]
            Specifies whether to include user-defined metadata in the search results.
        
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.
        
        Returns
        -------
        SearchResults
            Successfully performed a search request.
        
        Examples
        --------
        from twelvelabs import TwelveLabs
        
        client = TwelveLabs(
            api_key="YOUR_API_KEY",
        )
        client.search.create(
            index_id="index_id",
            search_options=["visual"],
        )
        """
        _response = self._raw_client.create(
            index_id=index_id,
            search_options=search_options,
            query_media_type=query_media_type,
            query_media_url=query_media_url,
            query_media_file=query_media_file,
            query_text=query_text,
            transcription_options=transcription_options,
            group_by=group_by,
            operator=operator,
            page_limit=page_limit,
            filter=filter,
            include_user_metadata=include_user_metadata,
            request_options=request_options,
        )
        return _response.data

    def retrieve(
        self,
        page_token: str,
        *,
        include_user_metadata: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SearchRetrieveResponse:
        """
        Use this endpoint to retrieve a specific page of search results.

        <Note title="Note">
        When you use pagination, you will not be charged for retrieving subsequent pages of results.
        </Note>

        Parameters
        ----------
        page_token : str
            A token that identifies the page to retrieve.

        include_user_metadata : typing.Optional[bool]
            Specifies whether to include user-defined metadata in the search results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SearchRetrieveResponse
            Successfully retrieved the specified page of search results.

        Examples
        --------
        from twelvelabs import TwelveLabs

        client = TwelveLabs(
            api_key="YOUR_API_KEY",
        )
        client.search.retrieve(
            page_token="1234567890",
            include_user_metadata=True,
        )
        """
        _response = self._raw_client.retrieve(
            page_token, include_user_metadata=include_user_metadata, request_options=request_options
        )
        return _response.data


class AsyncSearchClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawSearchClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawSearchClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawSearchClient
        """
        return self._raw_client

    async def create(
        self,
        *,
        index_id: str,
        search_options: typing.List[SearchCreateRequestSearchOptionsItem],
        query_media_type: typing.Optional[SearchCreateRequestQueryMediaType] = OMIT,
        query_media_url: typing.Optional[str] = OMIT,
        query_media_file: typing.Optional[core.File] = OMIT,
        query_text: typing.Optional[str] = OMIT,
        transcription_options: typing.Optional[typing.List[SearchCreateRequestTranscriptionOptionsItem]] = OMIT,
        group_by: typing.Optional[SearchCreateRequestGroupBy] = OMIT,
        operator: typing.Optional[SearchCreateRequestOperator] = OMIT,
        page_limit: typing.Optional[int] = OMIT,
        filter: typing.Optional[str] = OMIT,
        include_user_metadata: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SearchResults:
        """
        Use this endpoint to search for relevant matches in an index using text, media, or a combination of both as your query.
        
        **Text queries**:
        - Use the `query_text` parameter to specify your query.
        
        **Media queries**:
        - Set the `query_media_type` parameter to the corresponding media type (example: `image`).
        - Provide up to 10 images by specifying the following parameters multiple times:
          - `query_media_url`: Publicly accessible URL of your media file.
          - `query_media_file`: Local media file.
        **Composed text and media queries**:
        - Use the `query_text` parameter for your text query.
        - Set `query_media_type` to `image`.
        - Provide up to 10 images by specifying the `query_media_url` and `query_media_file` parameters multiple times.
        
        **Entity search** (beta):
        - To find a specific person in your videos, enclose the unique identifier of the entity you want to find in the `query_text` parameter.
        
        <Note title="Notes">
        - When using images in your search queries (either as media queries or in composed searches), ensure your image files meet the [requirements](/v1.3/docs/concepts/models/marengo#image-file-requirements).
        - This endpoint is rate-limited. For details, see the [Rate limits](/v1.3/docs/get-started/rate-limits) page.
        </Note>
        
        Parameters
        ----------
        index_id : str
            The unique identifier of the index to search.
        
        search_options : typing.List[SearchCreateRequestSearchOptionsItem]
            Specifies the modalities the video understanding model uses to find relevant information.
            
            Available options:
            - `visual`: Searches visual content.
            - `audio`: Searches non-speech audio.
            - `transcription`: Spoken words
            
            <Note title="Note">
            - You can specify multiple search options in conjunction with the [`operator`](/v1.3/api-reference/any-to-video-search/make-search-request#request.body.operator.operator) parameter described below to broaden or narrow your search. For example, to search using both visual and non-speech audio content, include this parameter two times in the request as shown below:
              ```JSON
              --form search_options=visual \
              --form search_options=audio \
              --form search_options=transcription \
              ```
            </Note>
            
            For guidance, see the [Search options](/v1.3/docs/concepts/modalities#search-options) section.
        
        query_media_type : typing.Optional[SearchCreateRequestQueryMediaType]
            The type of media you wish to use. This parameter is required for media queries. For example, to perform an image-based search, set this parameter to `image`. Use `query_text` together with this parameter when you want to perform a composed image+text search.
        
        query_media_url : typing.Optional[str]
            The publicly accessible URL of a media file to use as a query. This parameter is required for media queries if `query_media_file` is not provided.
            
            You can provide up to 10 images by specifying this parameter multiple times (Marengo 3.0 only):
            ```
            --form query_media_url=https://example.com/image1.jpg \
            --form query_media_url=https://example.com/image2.jpg
            ```
        
        query_media_file : typing.Optional[core.File]
            See core.File for more documentation
        
        query_text : typing.Optional[str]
            The text query to search for. This parameter is required for text queries. Note that the platform supports full natural language-based search. You can use this parameter together with `query_media_type` and `query_media_url` or `query_media_file` to perform a composed image+text search.
            
            If you're using the Entity Search feature to search for specific persons in your video content, you must enclose the unique identifier of your entity between the `<@` and `>` markers. For example, to search for an entity with the ID `entity123`, use `<@entity123> is walking` as your query.
            
            
            Marengo supports up to 500 tokens per query.
        
        transcription_options : typing.Optional[typing.List[SearchCreateRequestTranscriptionOptionsItem]]
            Specifies how the platform matches your text query with the words spoken in the video. This parameter applies only when the `search_options` parameter contains the `transcription` value.
            
            Available options:
            - `lexical`: Exact word matching
            - `semantic`: Meaning-based matching
            
            For details on when to use each option, see the [Transcription options](/v1.3/docs/concepts/modalities#transcription-options) section.
            
            **Default**: `["lexical", "semantic"]`.
        
        group_by : typing.Optional[SearchCreateRequestGroupBy]
            Use this parameter to group or ungroup items in a response. It can take one of the following values:
            - `video`:  The platform will group the matching video clips in the response by video.
            - `clip`: The matching video clips in the response will not be grouped.
            
            **Default:** `clip`
        
        operator : typing.Optional[SearchCreateRequestOperator]
            Combines multiple search options using `or` or `and`. Use `and` to find segments matching all search options. Use `or` to find segments matching any search option. For detailed guidance on using this parameter, see the [Combine multiple modalities](/v1.3/docs/concepts/modalities#combine-multiple-modalities) section.
            
            **Default**: `or`.
        
        page_limit : typing.Optional[int]
            The number of items to return on each page. When grouping by video, this parameter represents the number of videos per page. Otherwise, it represents the maximum number of video clips per page.
            
            **Max**: `50`.
        
        filter : typing.Optional[str]
            Specifies a stringified JSON object to filter your search results. Supports both system-generated metadata (example: video ID, duration) and user-defined metadata.
            
            **Syntax for filtering**
            
            The following table describes the supported data types, operators, and filter syntax:
            
            | Data type | Operator | Description | Syntax |
            |:----------|:---------|:------------|:-------|
            | String | `=` | Matches results equal to the specified value. | `{"field": "value"}`
            | Array of strings | `=` | Matches results with any value in the specified array. Supported only for `id`. | `{"id": ["value1", "value2"]}` |
            | Numeric (integer, float) | `=`, `lte`, `gte` | Matches results equal to or within a range of the specified value. | `{"field": number}` or `{"field": { "gte": number, "lte": number }}` |
            | Boolean | `=` | Matches results equal to the specified boolean value. | `{"field": true}` or `{"field": false}`. |
            
            <br/>
            **System-generated metadata**
            
            The table below describes the system-generated metadata available for filtering your  search results:
            
            | Field name | Description | Type | Example |
            |:-----------|:------------|:-----|:--------|
            | `id` | Filters by specific video IDs. | Array of strings | `{"id": ["67cec9caf45d9b64a58340fc", "67cec9baf45d9b64a58340fa"]}`. |
            | `duration` | Filters based on the duration of the video containing the segment that matches your query. | Number or object with `gte` and `lte` | `{"duration": 600}` or `{"duration": { "gte": 600, "lte": 800 }}` |
            | `width` | Filters by video width (in pixels). | Number or object with `gte` and `lte` | `{"width": 1920}` or `{"width": { "gte": 1280, "lte": 1920}}` |
            | `height` | Filters by video height (in pixels). | Number or object with `gte` and `lte`. | `{"height": 1080}` or `{"height": { "gte": 720, "lte": 1080 }}`. |
            | `size` | Filters by video size (in bytes) | Number or object with `gte` and `lte`. | `{"size": 1048576}` or `{"size": { "gte": 1048576, "lte": 5242880}}` |
            | `filename` | Filters by the exact file name. | String | `{"filename": "Animal Encounters part 1"}` |
            
            <br/>
            **User-defined metadata**
            
            To filter by user-defined metadata:
            1. Add metadata to your video by calling the [`PUT`](/v1.3/api-reference/videos/update) method of the `/indexes/:index-id/videos/:video-id` endpoint
            2. Reference the custom field in your filter object. For example, to filter videos where a custom field named `needsReview` of type boolean is `true`, use `{"needs_review": true}`.
            
            For more details and examples, see the [Filter search results](/v1.3/docs/guides/search/filtering) page.
        
        include_user_metadata : typing.Optional[bool]
            Specifies whether to include user-defined metadata in the search results.
        
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.
        
        Returns
        -------
        SearchResults
            Successfully performed a search request.
        
        Examples
        --------
        import asyncio
        
        from twelvelabs import AsyncTwelveLabs
        
        client = AsyncTwelveLabs(
            api_key="YOUR_API_KEY",
        )
        
        
        async def main() -> None:
            await client.search.create(
                index_id="index_id",
                search_options=["visual"],
            )
        
        
        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            index_id=index_id,
            search_options=search_options,
            query_media_type=query_media_type,
            query_media_url=query_media_url,
            query_media_file=query_media_file,
            query_text=query_text,
            transcription_options=transcription_options,
            group_by=group_by,
            operator=operator,
            page_limit=page_limit,
            filter=filter,
            include_user_metadata=include_user_metadata,
            request_options=request_options,
        )
        return _response.data

    async def retrieve(
        self,
        page_token: str,
        *,
        include_user_metadata: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SearchRetrieveResponse:
        """
        Use this endpoint to retrieve a specific page of search results.

        <Note title="Note">
        When you use pagination, you will not be charged for retrieving subsequent pages of results.
        </Note>

        Parameters
        ----------
        page_token : str
            A token that identifies the page to retrieve.

        include_user_metadata : typing.Optional[bool]
            Specifies whether to include user-defined metadata in the search results.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SearchRetrieveResponse
            Successfully retrieved the specified page of search results.

        Examples
        --------
        import asyncio

        from twelvelabs import AsyncTwelveLabs

        client = AsyncTwelveLabs(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.search.retrieve(
                page_token="1234567890",
                include_user_metadata=True,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.retrieve(
            page_token, include_user_metadata=include_user_metadata, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ....core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ....core.serialization import FieldMetadata


class TasksCreateResponse(UniversalBaseModel):
    id: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="_id")] = pydantic.Field(default=None)
    """
    The unique identifier of the video embedding task. You can use the identifier to:
    - Retrieve the status of your task by invoking the [`GET`](/v1.3/api-reference/create-embeddings-v1/video-embeddings/retrieve-video-embedding-task-status) method of the `/embed/tasks/{task_id}/status` endpoint.
    - Retrieve the embedding by invoking the  [`GET`](/v1.3/api-reference/create-embeddings-v1/video-embeddings/retrieve-video-embeddings) method of the `/embed/tasks/{task_id}` endpoint.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from .base_embedding_metadata import BaseEmbeddingMetadata


class AudioEmbeddingMetadata(BaseEmbeddingMetadata):
    """
    An object containing metadata associated with the embedding.
    """

    audio_embedding_options: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The type of the embedding. It can take one of the following values: ['audio'] or ['transcription'].
    """

    audio_embedding_scopes: typing.Optional[typing.List[str]] = pydantic.Field(default=None)
    """
    The scope you've specified in the request.
    """

    duration: typing.Optional[float] = pydantic.Field(default=None)
    """
    The total duration of the audio in seconds.
    """

    start_offset_sec: typing.Optional[float] = pydantic.Field(default=None)
    """
    The start offset in seconds from the beginning of the audio where processing should begin.
    """

    end_offset_sec: typing.Optional[float] = pydantic.Field(default=None)
    """
    The end offset in seconds from the beginning of the audio where processing should end.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



import typing
import time
import asyncio
from ..core.client_wrapper import SyncClientWrapper, AsyncClientWrapper
from ..embed.client import EmbedClient, AsyncEmbedClient
from ..embed.tasks.client import TasksClient, AsyncTasksClient
from ..embed.tasks.types.tasks_create_response import TasksCreateResponse
from ..embed.tasks.types.tasks_status_response import TasksStatusResponse
from ..embed.tasks.types.tasks_create_request_video_embedding_scope_item import (
    TasksCreateRequestVideoEmbeddingScopeItem,
)
from ..core.request_options import RequestOptions
from .. import core

OMIT = typing.cast(typing.Any, ...)


class CreateEmbeddingsTaskVideoParams(typing.TypedDict, total=False):
    """Parameters for creating a video embedding task."""

    video_file: typing.Optional[core.File]
    video_url: typing.Optional[str]
    video_start_offset_sec: typing.Optional[float]
    video_end_offset_sec: typing.Optional[float]
    video_clip_length: typing.Optional[int]
    video_embedding_scope: typing.Optional[
        typing.List[TasksCreateRequestVideoEmbeddingScopeItem]
    ]


class EmbedTasksClientWrapper(TasksClient):
    """Wrapper for the TasksClient that adds additional functionality."""

    def __init__(self, client_wrapper: SyncClientWrapper):
        """Initialize the EmbedTasksClientWrapper."""
        super().__init__(client_wrapper=client_wrapper)

    def create_bulk(
        self,
        *,
        model_name: typing.Literal["Marengo-retrieval-2.7"],
        videos: typing.List[CreateEmbeddingsTaskVideoParams],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[TasksCreateResponse]:
        """
        This method creates multiple video embedding tasks in bulk.

        Parameters
        ----------
        model_name : typing.Literal["Marengo-retrieval-2.7"]
            The name of the embedding model to use.

        videos : typing.List[CreateEmbeddingsTaskVideoParams]
            A list of video parameters for creating embedding tasks. Each item should have:
            - video_file or video_url: The video file or URL to embed
            - Optional parameters like start/end offsets, clip length, and embedding scopes

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[TasksCreateResponse]
            A list of video embedding tasks that were successfully created.

        Examples
        --------
        from twelvelabs import TwelveLabs
        from twelvelabs.wrapper.embed_client_wrapper import CreateEmbeddingsTaskVideoParams

        client = TwelveLabs(
            api_key="YOUR_API_KEY",
        )

        videos = [
            CreateEmbeddingsTaskVideoParams(
                video_url="https://example.com/video1.mp4",
                video_embedding_scopes=["clip", "video"]
            ),
            CreateEmbeddingsTaskVideoParams(
                video_url="https://example.com/video2.mp4",
                video_embedding_scopes=["clip"]
            )
        ]

        tasks = client.embed.tasks.create_bulk(
            model_name="Marengo-retrieval-2.7",
            videos=videos,
        )
        """
        tasks = []

        for video_params in videos:
            try:
                # Extract parameters, handling both dict and object access
                def get_param(key: str) -> typing.Any:
                    if hasattr(video_params, key):
                        return getattr(video_params, key)
                    return (
                        video_params.get(key)
                        if isinstance(video_params, dict)
                        else None
                    )

                # Build kwargs, only including non-None values
                kwargs: typing.Dict[str, typing.Any] = {"model_name": model_name}

                video_file = get_param("video_file")
                if video_file is not None:
                    kwargs["video_file"] = video_file

                video_url = get_param("video_url")
                if video_url is not None:
                    kwargs["video_url"] = video_url

                video_start_offset_sec = get_param("video_start_offset_sec")
                if video_start_offset_sec is not None:
                    kwargs["video_start_offset_sec"] = video_start_offset_sec

                video_end_offset_sec = get_param("video_end_offset_sec")
                if video_end_offset_sec is not None:
                    kwargs["video_end_offset_sec"] = video_end_offset_sec

                video_clip_length = get_param("video_clip_length")
                if video_clip_length is not None:
                    kwargs["video_clip_length"] = video_clip_length

                video_embedding_scope = get_param("video_embedding_scope")
                if video_embedding_scope is not None:
                    kwargs["video_embedding_scope"] = video_embedding_scope

                if request_options is not None:
                    kwargs["request_options"] = request_options

                task = self.create(**kwargs)
                tasks.append(task)
            except Exception as e:
                print(f"Error creating embedding task: {e}")
                continue

        return tasks

    def wait_for_done(
        self,
        task_id: str,
        *,
        sleep_interval: float = 5.0,
        callback: typing.Optional[typing.Callable[[TasksStatusResponse], None]] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TasksStatusResponse:
        """
        Wait for a task to complete by periodically checking its status.

        Parameters
        ----------
        task_id : str
            The unique identifier of the task to wait for.

        sleep_interval : float, optional
            The time in seconds to wait between status checks, by default 5.0

        callback : typing.Optional[typing.Callable[[TasksStatusResponse], None]], optional
            A function to call after each status check with the task response, by default None

        Returns
        -------
        TasksStatusResponse
            The completed task response

        Examples
        --------
        from twelvelabs import TwelveLabs

        client = TwelveLabs(
            api_key="YOUR_API_KEY",
        )

        task = client.embed.tasks.create(
            model_name="Marengo-retrieval-2.7",
            video_url="https://example.com/video.mp4",
        )

        completed_task = client.embed.tasks.wait_for_done(
            task_id=task._id,
            sleep_interval=10.0,
        )
        """
        if sleep_interval <= 0:
            raise ValueError("sleep_interval must be greater than 0")

        # Get initial task
        task = self.status(task_id, request_options=request_options)

        # Check if it's already done
        done_statuses = ["ready", "failed"]

        while task.status not in done_statuses:
            time.sleep(sleep_interval)

            try:
                task = self.status(task_id, request_options=request_options)
            except Exception as e:
                print(f"Retrieving task status failed: {e}. Retrying...")
                continue

            if callback is not None:
                callback(task)

        return task


class AsyncEmbedTasksClientWrapper(AsyncTasksClient):
    """Async wrapper for the TasksClient that adds additional functionality."""

    def __init__(self, client_wrapper: AsyncClientWrapper):
        """Initialize the AsyncEmbedTasksClientWrapper."""
        super().__init__(client_wrapper=client_wrapper)

    async def create_bulk(
        self,
        *,
        model_name: typing.Literal["Marengo-retrieval-2.7"],
        videos: typing.List[CreateEmbeddingsTaskVideoParams],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[TasksCreateResponse]:
        """
        This method creates multiple video embedding tasks in bulk.

        Parameters
        ----------
        model_name : typing.Literal["Marengo-retrieval-2.7"]
            The name of the embedding model to use.

        videos : typing.List[CreateEmbeddingsTaskVideoParams]
            A list of video parameters for creating embedding tasks. Each item should have:
            - video_file or video_url: The video file or URL to embed
            - Optional parameters like start/end offsets, clip length, and embedding scopes

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[TasksCreateResponse]
            A list of video embedding tasks that were successfully created.

        Examples
        --------
        import asyncio
        from twelvelabs import AsyncTwelveLabs
        from twelvelabs.wrapper.embed_client_wrapper import CreateEmbeddingsTaskVideoParams

        client = AsyncTwelveLabs(
            api_key="YOUR_API_KEY",
        )

        async def main() -> None:
            videos = [
                CreateEmbeddingsTaskVideoParams(
                    video_url="https://example.com/video1.mp4",
                    video_embedding_scopes=["clip", "video"]
                ),
                CreateEmbeddingsTaskVideoParams(
                    video_url="https://example.com/video2.mp4",
                    video_embedding_scopes=["clip"]
                )
            ]

            tasks = await client.embed.tasks.create_bulk(
                model_name="Marengo-retrieval-2.7",
                videos=videos,
            )
            print(tasks)

        asyncio.run(main())
        """
        tasks = []

        for video_params in videos:
            try:
                # Extract parameters, handling both dict and object access
                def get_param(key: str) -> typing.Any:
                    if hasattr(video_params, key):
                        return getattr(video_params, key)
                    return (
                        video_params.get(key)
                        if isinstance(video_params, dict)
                        else None
                    )

                # Build kwargs, only including non-None values
                kwargs: typing.Dict[str, typing.Any] = {"model_name": model_name}

                video_file = get_param("video_file")
                if video_file is not None:
                    kwargs["video_file"] = video_file

                video_url = get_param("video_url")
                if video_url is not None:
                    kwargs["video_url"] = video_url

                video_start_offset_sec = get_param("video_start_offset_sec")
                if video_start_offset_sec is not None:
                    kwargs["video_start_offset_sec"] = video_start_offset_sec

                video_end_offset_sec = get_param("video_end_offset_sec")
                if video_end_offset_sec is not None:
                    kwargs["video_end_offset_sec"] = video_end_offset_sec

                video_clip_length = get_param("video_clip_length")
                if video_clip_length is not None:
                    kwargs["video_clip_length"] = video_clip_length

                video_embedding_scope = get_param("video_embedding_scope")
                if video_embedding_scope is not None:
                    kwargs["video_embedding_scope"] = video_embedding_scope

                if request_options is not None:
                    kwargs["request_options"] = request_options

                task = await self.create(**kwargs)
                tasks.append(task)
            except Exception as e:
                print(f"Error creating embedding task: {e}")
                continue

        return tasks

    async def wait_for_done(
        self,
        task_id: str,
        *,
        sleep_interval: float = 5.0,
        callback: typing.Optional[
            typing.Callable[[TasksStatusResponse], typing.Awaitable[None]]
        ] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TasksStatusResponse:
        """
        Wait for a task to complete by periodically checking its status.

        Parameters
        ----------
        task_id : str
            The unique identifier of the task to wait for.

        sleep_interval : float, optional
            The time in seconds to wait between status checks, by default 5.0

        callback : typing.Optional[typing.Callable[[TasksStatusResponse], typing.Awaitable[None]]], optional
            A function to call after each status check with the task response, by default None

        Returns
        -------
        TasksStatusResponse
            The completed task response

        Examples
        --------
        import asyncio
        from twelvelabs import AsyncTwelveLabs

        client = AsyncTwelveLabs(
            api_key="YOUR_API_KEY",
        )

        task = await client.embed.tasks.create(
            model_name="Marengo-retrieval-2.7",
            video_url="https://example.com/video.mp4",
        )

        completed_task = await client.embed.tasks.wait_for_done(
            task_id=task._id,
            sleep_interval=10.0,
        )
        """
        if sleep_interval <= 0:
            raise ValueError("sleep_interval must be greater than 0")

        # Get initial task
        task = await self.status(task_id, request_options=request_options)

        # Check if it's already done
        done_statuses = ["ready", "failed"]

        while task.status not in done_statuses:
            await asyncio.sleep(sleep_interval)

            try:
                task = await self.status(task_id, request_options=request_options)
            except Exception as e:
                print(f"Retrieving task status failed: {e}. Retrying...")
                continue

            if callback is not None:
                await callback(task)

        return task


class EmbedClientWrapper(EmbedClient):
    """Wrapper for the EmbedClient that adds custom functionality."""

    def __init__(self, client_wrapper: SyncClientWrapper):
        """Initialize the EmbedClientWrapper."""
        super().__init__(client_wrapper=client_wrapper)
        # Replace the tasks property with our custom implementation
        self.tasks = EmbedTasksClientWrapper(client_wrapper=client_wrapper)


class AsyncEmbedClientWrapper(AsyncEmbedClient):
    """Async wrapper for the EmbedClient that adds custom functionality."""

    def __init__(self, client_wrapper: AsyncClientWrapper):
        """Initialize the AsyncEmbedClientWrapper."""
        super().__init__(client_wrapper=client_wrapper)
        # Replace the tasks property with our custom implementation
        self.tasks = AsyncEmbedTasksClientWrapper(client_wrapper=client_wrapper)



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from . import tasks, v_2
from .tasks import (
    TasksCreateRequestVideoEmbeddingScopeItem,
    TasksCreateResponse,
    TasksListResponse,
    TasksListResponsePageInfo,
    TasksRetrieveRequestEmbeddingOptionItem,
    TasksRetrieveResponse,
    TasksRetrieveResponseVideoEmbedding,
    TasksStatusResponse,
    TasksStatusResponseVideoEmbedding,
)
from .v_2 import CreateEmbeddingsRequestInputType, CreateEmbeddingsRequestModelName

__all__ = [
    "CreateEmbeddingsRequestInputType",
    "CreateEmbeddingsRequestModelName",
    "TasksCreateRequestVideoEmbeddingScopeItem",
    "TasksCreateResponse",
    "TasksListResponse",
    "TasksListResponsePageInfo",
    "TasksRetrieveRequestEmbeddingOptionItem",
    "TasksRetrieveResponse",
    "TasksRetrieveResponseVideoEmbedding",
    "TasksStatusResponse",
    "TasksStatusResponseVideoEmbedding",
    "tasks",
    "v_2",
]
